local sti = require "libs/sti"
local softbody = require "libs/loveballs"

local origin = {}
local scale = 2

function love.load()
  -- uncomment the line below if using zerobrane debugging (http://notebook.kulchenko.com/zerobrane/love2d-debugging)
  -- if arg[#arg] == "-debug" then require("mobdebug").start() end
  
  map = sti("assets/maps/mymap.lua", {"box2d"})
  origin.x, origin.y = 50, 50

  -- physics example derived from love2d physics tutorial (https://love2d.org/wiki/Tutorial:Physics)

  love.physics.setMeter(64)
  world = love.physics.newWorld(0, 9.81*64, true)

  objects = {}
  objects.ball = {}
  
  -- softbody physics provided by loveballs lib (https://github.com/exezin/loveballs)
  
  objects.ball.body = softbody(world, origin.x, origin.y, 20)
  objects.ball.shape = love.physics.newCircleShape(20)
  objects.ball.fixture = love.physics.newFixture(objects.ball.body.centerBody, objects.ball.shape, 1)
  map:box2d_init(world)
end


function love.update(dt)

  map:update(dt)
	objects.ball.body:update(dt)

  if love.keyboard.isDown("right") then
    objects.ball.body.centerBody:applyForce(400, 0)
  elseif love.keyboard.isDown("left") then
    objects.ball.body.centerBody:applyForce(-400, 0)
  elseif love.keyboard.isDown("down") then
    objects.ball.body.centerBody:applyForce(-0, 400)
	elseif love.keyboard.isDown("space") then
		objects.ball.body.centerBody:applyForce(0, -2000)
  elseif love.keyboard.isDown("up") then
    objects.ball.body.centerBody:setPosition(50, origin.y)
    objects.ball.body.centerBody:setLinearVelocity(0, 0)
  end

  world:update(dt)
end

function love.draw()
  
  dx = objects.ball.body.centerBody:getX() - (love.graphics.getWidth() / 2) / scale
  dy = objects.ball.body.centerBody:getY() - (love.graphics.getHeight() / 2) / scale

  love.graphics.scale(scale)
  love.graphics.translate(-dx, -dy)

	objects.ball.body:draw()
  map:draw(-dx, -dy, scale, scale)
  
end
